%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [phi, dIdz_est, Psi] = TIE_solver_meanI(dIdz,I0z,pixelsize,k,r,thr,method)
% function phi = TIE_solver_meanI(dIdz,I0z,pixelsize,k,r,threshold)
% Purpose: Solution to TIE using FFT-based Possion solver
%'Inputs': 'dIdz',Intensity derivative along optical axis;
%          'I0z', Infoucs intensity image 
%          'pixelsize'pixelsize('mm'); 
%          'k', Wavenumber
%          'r', regularzation parameter (to remove low-frequency noise)
%          'thr', intensity threshold to prevent intensity zeros (%
%          of the maximam value of the intensity distribution (recommended
%          value 0.01)
%          'method': 'FFT' or 'DCT' or 'ZeroPadding';
%'Outputs': 'phi',object phase map;
%           'dIdz_est', estimated dIdz;
%           'Psi', Psi;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Version 2.0 - 
% Coded by Chao Zuo - 2013-11-19   
% Version 2.2 - 
% Modified by Lei - 2014-08-15   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check nargin.............................................................

switch nargin
    case 6
        method = 'DCT';
	case 5
        method = 'DCT';
        thr = 0.01;
    case 4
        method = 'DCT';
        thr = 0.01;
        r = eps;
end

if isempty(thr)
    thr = 0.01;
end

if isempty(r)
    r = eps;
end

% Check method.............................................................

J = -k*dIdz;

if strcmpi(method, 'DCT')
    % EvenFlip Matrices
    J = EvenFlip(J);
    I0z = EvenFlip(I0z);
elseif strcmpi(method, 'ZeroPaddingFFT')
    % MeanPad Matrices
    J = ZeroPadding(J);
    I0z = ZeroPadding(I0z);
elseif strcmpi(method, 'FFT')
    % Do nothing.
else
    error('Method can only be ''FFT'', ''DCT'', or ''ZeroPadding'' for now')
end

% Coordinates in frequency domain..........................................
[Ny,Nx] = size(J);
nx = 1:Nx;
ny = 1:Ny;
u = ((nx-1)-Nx/2)/(pixelsize*Nx);
v = ((ny-1)-Ny/2)/(pixelsize*Ny);
[fx,fy] = meshgrid(u,v); 
fx = fftshift(fx);
fy = fftshift(fy);

% Terms for derivative calculation in frequency domain.....................
Cx = 2*1i*pi*fx;
Cy = 2*1i*pi*fy;

% Calculate Psi(optional), the main perpose is to calculate FPsi...........
FJ = fft2(J);
FPsi = FJ.*(Cx.*Cx+Cy.*Cy)./(r+(Cx.*Cx+Cy.*Cy).^2);
Psi = real(ifft2(FPsi)); 
if strcmpi(method, 'DCT')
    Psi = Psi(1:end/2,1:end/2); % optional result.
elseif strcmpi(method, 'ZeroPaddingFFT')
    Psi = Psi(1:end/2,1:end/2); % optional result.
end

% Calculate phi............................................................ 
% Set the intensity out of aperture as the mean value of intensity inside.
OutAper = I0z<thr*max(max(I0z));
Im = I0z;
Im(OutAper) = mean(I0z(~OutAper));

% From (dPsidx, dPsidy) to (dphidx, dphidy).
dPsidx = real(ifft2(FPsi.*Cx));
dPsidy = real(ifft2(FPsi.*Cy));
dphidx = dPsidx./Im;
dphidy = dPsidy./Im;

% Calculate the laplace of phi
Fdphidx = fft2(dphidx);
% d2phidx2 = real(ifft2(Fdphidx.*Cx));
Fdphidy = fft2(dphidy);
% d2phidy2 = real(ifft2(Fdphidy.*Cy));
% laplacephi = d2phidx2+d2phidy2;
% 
% % Integration from laplace.
% Flaplacephi = fft2(laplacephi);
% Fphi = Flaplacephi.*(Cx.*Cx+Cy.*Cy)./(r+(Cx.*Cx+Cy.*Cy).^2);

Fphi = (Fdphidx.*Cx+Fdphidy.*Cy).*(Cx.*Cx+Cy.*Cy)./(r+(Cx.*Cx+Cy.*Cy).^2);
phi = real(ifft2(Fphi));
if strcmpi(method, 'DCT')
    phi = phi(1:end/2,1:end/2); % result.
elseif strcmpi(method, 'ZeroPaddingFFT')
    phi = phi(1:end/2,1:end/2); % result.
end


Fdphidx = Fphi.*Cx;
Fdphidy = Fphi.*Cy;
dphidx = real(ifft2(Fdphidx));
dphidy = real(ifft2(Fdphidy));

dPsidx = I0z.*dphidx;
dPsidy = I0z.*dphidy;

FdPsidx = fft2(dPsidx);
FdPsidy = fft2(dPsidy);

Fd2Psidx2 = FdPsidx.*Cx;
Fd2Psidy2 = FdPsidy.*Cy;

d2Psidx2 = real(ifft2(Fd2Psidx2));
d2Psidy2 = real(ifft2(Fd2Psidy2));

laplacePsi = d2Psidx2+d2Psidy2;

dIdz_est = laplacePsi./(-k);
if strcmpi(method, 'DCT')
    dIdz_est = dIdz_est(1:end/2,1:end/2); % optional result.
elseif strcmpi(method, 'ZeroPaddingFFT')
    dIdz_est = dIdz_est(1:end/2,1:end/2); % optional result.
end

end


% subfunctions.
function AA = EvenFlip(A)

temp = [A,fliplr(A)];
AA = [temp',(flipud(temp))']';

end

function AA = ZeroPadding(A)

AA = [A,A*0;A*0,A*0];

end
