% DemoScript for TIE with arbitrary aperture.

%% Clear everything existing.

clear all;
close all;
clc;

addpath('..\TIE Functions');

%% Setting up system parameters in simulation.

% Set the size of CCD.
Nx = 256;
Ny = Nx;

pixelsize = 2e-6;   % 2um pixel size (m)
lambda = 633e-9;    % wavelength (m)
k = 2*pi/lambda;    % wave number
dz = 500e-6;       	% defocus distance (m)

Flag_Intensity = 0;	% 1: uniform intensity
                	% 0: nonuniform intensity distribution

Flag_Hann = 0;      % 1: Use Hann distribution phase object.
                    % 0: Do not use Hann phase object.

Flag_AperShape = 'telescope-';    
% 'rect': Rect.
% 'circle': R=0.65*Rmax Aperture.
% 'circle-': circle-.
% 'ellipse': ellipse.
% 'ellipse-': ellipse-.
% 'telescope-': telescope-.
% 'telescope': telescope.
% 'full': Do not use Aperture.


sigma = 1.8e-4;             % sigma of Gaussian intensity distribution (m)

% Symbolize expression.
syms Phase_sym Inten_sym x y
Inten_sym = exp((-x.^2-y.^2)/(2*sigma.^2));
Phase_sym = 0.82+(-0.7*(x*1e3)+(x*1e3).^2*10-(y*1e3).^2*10+2*(y*1e3));

%% Generate intensity and phase distribution.

eval GenerateOpticalField_DoubleSizeImplementation;

%% Numerical propagation to generate defocused images.

% Complex amplitude of optical field in foucs.
U0z = sqrt(I0z).*exp(1i.*phi);

% Numerical propagation for defocused images.
Unz = Numerical_Propagation(U0z,-dz,pixelsize,lambda,'Angular Spectrum');
Upz = Numerical_Propagation(U0z,+dz,pixelsize,lambda,'Angular Spectrum');

% Cut the centrial region which is able to be captured by CCD.
Indx = Nx_LF/4+1:Nx_LF/4+Nx;
Indy = Ny_LF/4+1:Ny_LF/4+Ny;
I0z = I0z(Indy,Indx);
Inz = Unz(Indy,Indx).*conj(Unz(Indy,Indx));
Ipz = Upz(Indy,Indx).*conj(Upz(Indy,Indx));
phi = phi(Indy,Indx);
Aperture = Aperture(Indy,Indx);
phi(~Aperture) = NaN;

% Axial intensity derivative.
dIdz = (Ipz-Inz)/2/dz; 

% Check if satisfying Conservation of Energy.
disp(['mean2(dIdz)) = ' num2str(mean2(dIdz))]);
if abs(mean2(dIdz))>1e-7    % Need to check the threshold (1e-7 for now!)
    warning('Conservation of Energy may not be satisfied!');
end

%% Show the images.

% Show Intensities.
I_show = [mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)];
figure('Name','[mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)]');
imshow(I_show);
pause(0.05)

%% Solve TIE with iteration function.

NumerProp.dz = dz;
NumerProp.lambda = lambda;

RegPara = eps;
IntThr = 0.01;
JudgeFlag = 'delta_dIdzStd';
StdThr = 0.05;
MaxIterNum = 30;

% .........................................................................
% DCT......................................................................
% .........................................................................
method = 'DCT';

% phi_NumerProp_DCT........................................................
phi_NumerProp_DCT = TIEiter(dIdz,I0z,pixelsize,k,RegPara,IntThr,method ...
    , MaxIterNum, JudgeFlag, StdThr, NumerProp);
% Error Calculation.
phiErr_NumerProp_DCT = RemovePiston(phi_NumerProp_DCT - phi);


% phi_TIE_DCT..............................................................
phi_TIE_DCT = TIEiter(dIdz,I0z,pixelsize,k,RegPara,IntThr,method ...
    , MaxIterNum, JudgeFlag, StdThr);
% Error Calculation.
phiErr_TIE_DCT = RemovePiston(phi_TIE_DCT - phi);


% .........................................................................
% FFT......................................................................
% .........................................................................
method = 'FFT';

% phi_NumerProp_FFT........................................................
phi_NumerProp_FFT = TIEiter(dIdz,I0z,pixelsize,k,RegPara,IntThr,method ...
    , MaxIterNum, JudgeFlag, StdThr, NumerProp);
% Error Calculation.
phiErr_NumerProp_FFT = RemovePiston(phi_NumerProp_FFT - phi);

% phi_TIE_FFT..............................................................
phi_TIE_FFT = TIEiter(dIdz,I0z,pixelsize,k,RegPara,IntThr,method ...
    , MaxIterNum, JudgeFlag, StdThr);
% Error Calculation.
phiErr_TIE_FFT = RemovePiston(phi_TIE_FFT - phi);


% .........................................................................
% ZeroPaddingFFT...........................................................
% .........................................................................
method = 'ZeroPaddingFFT';

% phi_NumerProp_ZeroPaddingFFT.............................................
phi_NumerProp_ZeroPaddingFFT = TIEiter(dIdz,I0z,pixelsize,k,RegPara,IntThr,method ...
    , MaxIterNum, JudgeFlag, StdThr, NumerProp);
% Error Calculation.
phiErr_NumerProp_ZeroPaddingFFT = RemovePiston(phi_NumerProp_ZeroPaddingFFT - phi);

% phi_TIE_ZeroPaddingFFT...................................................
phi_TIE_ZeroPaddingFFT = TIEiter(dIdz,I0z,pixelsize,k,RegPara,IntThr,method ...
    , MaxIterNum, JudgeFlag, StdThr);
% Error Calculation.
phiErr_TIE_ZeroPaddingFFT = RemovePiston(phi_TIE_ZeroPaddingFFT - phi);

%% Show the result.

M = 2;
N = 6;
MP = get(0,'MonitorPositions');
figure('Position',[MP(3:4)/6,MP(3:4)/3*2]);
subplot(M,N,1);
imshow(phi_NumerProp_DCT,[]);
title('phi NumerProp DCT');
colorbar;

subplot(M,N,2);
imshow(phi_TIE_DCT,[]);
title('phi TIE DCT');
colorbar;

subplot(M,N,3);
imshow(phi_NumerProp_FFT,[]);
title('phi NumerProp FFT');
colorbar;

subplot(M,N,4);
imshow(phi_TIE_FFT,[]);
title('phi TIE FFT');
colorbar;

subplot(M,N,5);
imshow(phi_NumerProp_ZeroPaddingFFT,[]);
title('phi NumerProp ZeroPaddingFFT');
colorbar;

subplot(M,N,6);
imshow(phi_TIE_ZeroPaddingFFT,[]);
title('phi TIE ZeroPaddingFFT');
colorbar;

subplot(M,N,N+1);
imshow(phiErr_NumerProp_DCT,[]);
colormap(jet);
title('phiErr NumerProp DCT');
colorbar;

subplot(M,N,N+2);
imshow(phiErr_TIE_DCT,[]);
colormap(jet);
title('phiErr TIE DCT');
colorbar;

subplot(M,N,N+3);
imshow(phiErr_NumerProp_FFT,[]);
colormap(jet);
title('phiErr NumerProp FFT');
colorbar;

subplot(M,N,N+4);
imshow(phiErr_TIE_FFT,[]);
colormap(jet);
title('phiErr TIE FFT');
colorbar;

subplot(M,N,N+5);
imshow(phiErr_NumerProp_ZeroPaddingFFT,[]);
colormap(jet);
title('phiErr NumerProp ZeroPaddingFFT');
colorbar;

subplot(M,N,N+6);
imshow(phiErr_TIE_ZeroPaddingFFT,[]);
colormap(jet);
title('phiErr TIE ZeroPaddingFFT');
colorbar;
