% DemoScript for TIE with arbitrary aperture.

%% Clear everything existing.

clear all;
close all;
clc;

addpath('..\TIE Functions');

%% Setting up system parameters in simulation.

% Set the size of CCD.
Nx = 256;
Ny = Nx;

pixelsize = 2e-6;   % 2um pixel size (m)
lambda = 633e-9;    % wavelength (m)
k = 2*pi/lambda;    % wave number
dz = 500e-6;       	% defocus distance (m)

Flag_Intensity = 0;	% 1: uniform intensity
                	% 0: nonuniform intensity distribution

Flag_Hann = 0;      % 1: Use Hann distribution phase object.
                    % 0: Do not use Hann phase object.

Flag_AperShape = 'telescope-';    
% 'rect': Rect.
% 'circle': R=0.65*Rmax Aperture.
% 'circle-': circle-.
% 'ellipse': ellipse.
% 'ellipse-': ellipse-.
% 'telescope-': telescope-.
% 'telescope': telescope.
% 'full': Do not use Aperture.

sigma = 1.8e-4;     % sigma of Gaussian intensity distribution (m)

% Symbolize expression.
syms Phase_sym Inten_sym x y
Inten_sym = exp((-x.^2-y.^2)/(2*sigma.^2));
Phase_sym = 0.82+(-0.7*(x*1e3)+(x*1e3).^2*10-(y*1e3).^2*10+2*(y*1e3));

%% Generate intensity and phase distribution.

eval GenerateOpticalField_DoubleSizeImplementation;


%% Numerical propagation to generate defocused images.

% Complex amplitude of optical field in foucs.
U0z = sqrt(I0z).*exp(1i.*phi);

% Numerical propagation for defocused images.
Unz = Numerical_Propagation(U0z,-dz,pixelsize,lambda,'Angular Spectrum');
Upz = Numerical_Propagation(U0z,+dz,pixelsize,lambda,'Angular Spectrum');

% Cut the centrial region which is able to be captured by CCD.
Indx = Nx_LF/4+1:Nx_LF/4+Nx;
Indy = Ny_LF/4+1:Ny_LF/4+Ny;
I0z = I0z(Indy,Indx);
Inz = Unz(Indy,Indx).*conj(Unz(Indy,Indx));
Ipz = Upz(Indy,Indx).*conj(Upz(Indy,Indx));
phi = phi(Indy,Indx);
Aperture = Aperture(Indy,Indx);

x = x(Indy,Indx);
y = y(Indy,Indx);

figure;imagesc(x(1,:)*1e3,y(:,1)'*1e3,phi);
axis image; 
colorbar;
set(gca,'FontSize',14)

phi(~Aperture) = NaN;

% Add noise.
I0z = I0z + 0.000*randn(size(I0z));
Inz = Inz + 0.000*randn(size(Inz));
Ipz = Ipz + 0.000*randn(size(Ipz));


% Axial intensity derivative.
dIdz = (Ipz-Inz)/(2*dz); 

% Check if satisfying Conservation of Energy.
disp(['mean2(dIdz)) = ' num2str(mean2(dIdz))]);
if abs(mean2(dIdz))>1e-7    % Need to check the threshold (1e-7 for now!)
%     warndlg('Conservation of Energy may not be satisfied!');
end

%% Show the images.

% Show Intensities.
I_show = [mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)];
figure('Name','[mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)]');
imshow(I_show);
pause(0.05);

%% Solve TIE with iteration parameters.

RegPara = eps;
IntThr = 0.01;
method = 'DCT';
JudgeFlag = 'delta_dIdzStd';
StdThr = 1;
MaxIterNum = 10;

%% Solve TIE with iteration Script.

% TIE_solver
[phi_DCT_0, dIdz_0, Psi_0] = TIE_solver_meanI(dIdz,I0z,pixelsize,k,RegPara,IntThr,method);

% Error Calculation.
phiErr_DCT_0 = phi_DCT_0 - phi;
phiErr_DCT_0 = phiErr_DCT_0-mean(phiErr_DCT_0(~isnan(phiErr_DCT_0)));

% Iterative Compensation.

% initialization.
phi_DCT = phi_DCT_0;
phiErr_DCT = phiErr_DCT_0;
phi_comp = phi_DCT_0;
delta_dIdz = dIdz;
delta_dIdz_est = dIdz_0;

% Prepare the matrices.
IterNum = 1:MaxIterNum;
ErrStd = NaN*[0,IterNum];
ErrPV = NaN*[0,IterNum];
delta_dIdzStd = NaN*IterNum;
delta_dIdzPV = NaN*IterNum;
phi_compStd = NaN*IterNum;
phi_compPV = NaN*IterNum;
OptStdDiff = NaN*IterNum;


for num = IterNum

    % Check the std.
    ErrStd(num) = std(phiErr_DCT(~isnan(phiErr_DCT)));
    ErrPV(num) = range(phiErr_DCT(~isnan(phiErr_DCT)));

    dIdz_est = Estimate_dIdz(phi_DCT,I0z,pixelsize,k);
    delta_dIdz = dIdz - dIdz_est;

    % Check the std and PV.
    delta_dIdzStd(num) = std(delta_dIdz(~isnan(delta_dIdz)));
    delta_dIdzPV(num) = range(delta_dIdz(~isnan(delta_dIdz)));

    if num>1
        OptStd_last = OptStd;
    elseif num==1
        OptStd_last = 0;
    end

    % TIE_solver
    [phi_comp, ~ ] = TIE_solver_meanI(delta_dIdz,I0z,pixelsize,k,RegPara,IntThr,method);
    % Check the std and PV.
    phi_compStd(num) = std(phi_comp(~isnan(phi_comp)));
    phi_compPV(num) = range(phi_comp(~isnan(phi_comp)));
    
    OptStd = std(delta_dIdz_est(~isnan(delta_dIdz_est)))./std(phi_comp(~isnan(phi_comp)));
    OptStdDiff(num) = OptStd - OptStd_last;
    
    if strcmpi(JudgeFlag,'delta_dIdzStd')
        Judge = delta_dIdzStd(num)<=StdThr;
    elseif strcmpi(JudgeFlag,'OptStdDiff')
        Judge = OptStdDiff(num)<=0;
    end
    if Judge
        break;
    else
        % update to phi_DCT.
        phi_DCT = phi_DCT + phi_comp;
       
    end

    % Error Calculation.
    phiErr_DCT = phi_DCT - phi;
    phiErr_DCT = phiErr_DCT-mean(phiErr_DCT(~isnan(phiErr_DCT)));

end
% Check the std.
ErrStd(num+1) = std(phiErr_DCT(~isnan(phiErr_DCT)));
ErrPV(num+1) = range(phiErr_DCT(~isnan(phiErr_DCT)));

figure;plot([0,IterNum],ErrStd,'ro-',[0,IterNum],ErrPV,'bs-');
text(IterNum(1),ErrStd(1),'Std','VerticalAlignment','Bottom','Color','r');
text(IterNum(1),ErrPV(1),'PV','VerticalAlignment','Bottom','Color','b');
xlabel('Iterations');
ylabel('Error of phi');

figure;plot(IterNum,delta_dIdzStd,'ro-',IterNum,delta_dIdzPV,'bs-');
text(IterNum(1),delta_dIdzStd(1),'Std','VerticalAlignment','Bottom','Color','r');
text(IterNum(1),delta_dIdzPV(1),'PV','VerticalAlignment','Bottom','Color','b');
xlabel('Iterations');
ylabel('\Delta dIdz');

figure;plot(IterNum,phi_compStd,'ro-',IterNum,phi_compPV,'bs-');
text(IterNum(1),phi_compStd(1),'Std','VerticalAlignment','Bottom','Color','r');
text(IterNum(1),phi_compPV(1),'PV','VerticalAlignment','Bottom','Color','b');
xlabel('Iterations');
ylabel('phi comp');

figure;plot(IterNum,OptStdDiff,'ro-');
text(IterNum(1),OptStdDiff(1),'Std','VerticalAlignment','Bottom','Color','r');
xlabel('Iterations');
ylabel('Opt Std Diff');

%% Show the result.
FlagShow = 1;
if FlagShow == 1

M = 2;
N = 3;
MP = get(0,'MonitorPositions');
figure('Position',[MP(3:4)/6,MP(3:4)/3*2]);
subplot(M,N,1);
imshow(phi,[]);
title('phi');
colorbar;

subplot(M,N,2);
imshow(phi_DCT_0,[]);
title('phi DCT 0');
colorbar;

subplot(M,N,3);
imshow(phi_DCT,[]);
title('phi DCT');
colorbar;

subplot(M,N,N+2);
imshow(phiErr_DCT_0,[]);
colormap(jet);
title('phiErr DCT 0');
colorbar;

subplot(M,N,N+3);
imshow(phiErr_DCT,[]);
colormap(jet);
title('phiErr DCT');
colorbar;

%

figure;imagesc(x(1,:)*1e3,y(:,1)'*1e3,dIdz);
axis image; 
colormap hsv;
colorbar;
set(gca,'FontSize',14)

figure;imshow(dIdz,[]);
axis image; 
colormap gray;
colorbar;
set(gca,'FontSize',14)

J = -k*dIdz;
figure;imshow(J,[]);
colormap hsv;


figure;imshow(I0z,[0 1]);

figure;imshow(Inz,[0 1]);
figure;imshow(Ipz,[0 1]);


A = J;
temp = [A,fliplr(A)];
AA = [temp',(flipud(temp))']';
JJ = AA;
figure;imshow(JJ,[]);
colormap hsv;

I = I0z;
I(~Aperture) = mean(I0z(Aperture));
figure;imshow(I,[0 1]);

A = I;
temp = [A,fliplr(A)];
AA = [temp',(flipud(temp))']';
II = AA;
figure;imshow(II,[0 1]);


A = Psi_0;
temp = [A,fliplr(A)];
AA = [temp',(flipud(temp))']';
figure;imshow(AA,[]);
colormap jet;


A = phi_DCT_0;
temp = [A,fliplr(A)];
AA = [temp',(flipud(temp))']';
figure;imshow(AA,[]);
colormap jet;

figure;imshow(phi_DCT_0,[]);
colormap jet;

figure;imshow(Aperture,[]);

J_est = -k*dIdz_est;
figure;imshow(J_est,[]);
colormap hsv;

delta_J = -k*delta_dIdz;
figure;imshow(delta_J,[]);
colormap hsv;

figure;imshow(phi_comp,[]);
colormap jet;

figure;imshow(phi_DCT,[]);
colormap jet;

ResultantPhi = phi_DCT;
ResultantPhi(~Aperture)=NaN;
figure;imshow(ResultantPhi,[]);
colormap jet;


figure;plot([0,IterNum],ErrStd,'bo-','Linewidth',2);
xlabel('Iterations');
ylabel('Error of phi');
set(gca,'Fontsize',14,'Linewidth',2)

abs_delta_JStd = abs(-k*delta_dIdzStd);
figure;plot(IterNum,abs_delta_JStd,'ro-');
xlabel('Iterations');
ylabel('\Delta J');

figure;plot(IterNum,phi_compStd,'ro-');
xlabel('Iterations');
ylabel('phi comp');

figure;imagesc(x(1,:)*1e3,y(:,1)'*1e3,phiErr_DCT_0);
axis image; 
colormap(WRGBKColorMap2)
colorbar;
set(gca,'FontSize',14)
M = max(phiErr_DCT_0(~isnan(phiErr_DCT_0)));
m = min(phiErr_DCT_0(~isnan(phiErr_DCT_0)));
if abs(M)>abs(m)
    Mm = abs(M);
else
    Mm = abs(m);
end
caxis([-1 1]*Mm)

figure;imagesc(x(1,:)*1e3,y(:,1)'*1e3,phiErr_DCT);
axis image; 
colormap(WRGBKColorMap2)
colorbar;
set(gca,'FontSize',14);
caxis([-1 1]*Mm)


figure;imagesc(x(1,:)*1e3,y(:,1)'*1e3,phi_DCT);
axis image; 
colorbar;
set(gca,'FontSize',14);

figure;imagesc(x(1,:)*1e3,y(:,1)'*1e3,ResultantPhi);
axis image; 
colorbar;
set(gca,'FontSize',14);

end
