% DemoScript for TIE with arbitrary aperture.

%% Clear everything existing.

clear all;
close all;
clc;

addpath('..\TIE Functions');

%% Setting up system parameters in simulation.

% Set the size of CCD.
Nx = 256;
Ny = Nx;

pixelsize = 2e-6;   % 2um pixel size (m)
lambda = 633e-9;    % wavelength (m)
k = 2*pi/lambda;    % wave number
dz = 500e-6;       	% defocus distance (m)

Flag_Intensity = 0;	% 1: uniform intensity
                	% 0: nonuniform intensity distribution

Flag_Hann = 0;      % 1: Use Hann distribution phase object.
                    % 0: Do not use Hann phase object.

Flag_AperShape = 'telescope-';    
% 'rect': Rect.
% 'circle': R=0.65*Rmax Aperture.
% 'circle-': circle-.
% 'ellipse': ellipse.
% 'ellipse-': ellipse-.
% 'telescope-': telescope-.
% 'telescope': telescope.
% 'full': Do not use Aperture.

sigma = 1.8e-4;     % sigma of Gaussian intensity distribution (m)

% Symbolize expression.
syms Phase_sym Inten_sym x y
Inten_sym = exp((-x.^2-y.^2)/(2*sigma.^2));
Phase_sym = 0.82+(-0.7*(x*1e3)+(x*1e3).^2*10-(y*1e3).^2*10+2*(y*1e3));

%% Generate intensity and phase distribution.

eval GenerateOpticalField_DoubleSizeImplementation;

%% Numerical propagation to generate defocused images.

% Complex amplitude of optical field in foucs.
U0z = sqrt(I0z).*exp(1i.*phi);

% Numerical propagation for defocused images.
Unz = Numerical_Propagation(U0z,-dz,pixelsize,lambda,'Angular Spectrum');
Upz = Numerical_Propagation(U0z,+dz,pixelsize,lambda,'Angular Spectrum');

% Cut the centrial region which is able to be captured by CCD.
Indx = Nx_LF/4+1:Nx_LF/4+Nx;
Indy = Ny_LF/4+1:Ny_LF/4+Ny;
I0z = I0z(Indy,Indx);
Inz = Unz(Indy,Indx).*conj(Unz(Indy,Indx));
Ipz = Upz(Indy,Indx).*conj(Upz(Indy,Indx));
phi = phi(Indy,Indx);
Aperture = Aperture(Indy,Indx);
phi(~Aperture) = NaN;

% Add noise.
NOISES = 0:1e-3:4e-3;
phiErrStd = NaN(1,length(NOISES));
phiErr_DCT_Array = NaN(Ny,Nx*length(NOISES));
dIdz_Array = NaN(Ny,Nx*length(NOISES));
phi_DCT_Array = NaN(Ny,Nx*length(NOISES));
for NoiseNum = 1:length(NOISES)
    
    NoiseStd = NOISES(NoiseNum);
    I0z = I0z + NoiseStd*randn(size(I0z));
    Inz = Inz + NoiseStd*randn(size(Inz));
    Ipz = Ipz + NoiseStd*randn(size(Ipz));


    % Axial intensity derivative.
    dIdz = (Ipz-Inz)/(2*dz); 

%     % Show Intensities.
%     I_show = [mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)];
%     figure('Name','[mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)]');
%     imshow(I_show);
%     pause(0.05);

    % Solve TIE with iteration parameters.
    RegPara = eps;
    IntThr = 0.1;
    method = 'DCT';
    JudgeFlag = 'delta_dIdzStd';
    StdThr = 1e-3;
    MaxIterNum = 15;

    % Solve TIE with iteration Script.

    % TIE_solver
    [phi_DCT_0, dIdz_0] = TIE_solver_meanI(dIdz,I0z,pixelsize,k,RegPara,IntThr,method);

    % Error Calculation.
    phiErr_DCT_0 = phi_DCT_0 - phi;
    phiErr_DCT_0 = phiErr_DCT_0-mean(phiErr_DCT_0(~isnan(phiErr_DCT_0)));

    % Iterative Compensation.

    % initialization.
    phi_DCT = phi_DCT_0;
    phiErr_DCT = phiErr_DCT_0;
    phi_comp = phi_DCT_0;
    delta_dIdz = dIdz;
    delta_dIdz_est = dIdz_0;

    % Prepare the matrices.
    IterNum = 1:MaxIterNum;
    ErrStd = NaN*[0,IterNum];
    ErrPV = NaN*[0,IterNum];
    delta_dIdzStd = NaN*IterNum;
    delta_dIdzPV = NaN*IterNum;
    phi_compStd = NaN*IterNum;
    phi_compPV = NaN*IterNum;
    OptStdDiff = NaN*IterNum;


    for num = IterNum

        % Check the std.
        ErrStd(num) = std(phiErr_DCT(~isnan(phiErr_DCT)));
        ErrPV(num) = range(phiErr_DCT(~isnan(phiErr_DCT)));

        dIdz_est = Estimate_dIdz(phi_DCT,I0z,pixelsize,k);
        delta_dIdz = dIdz - dIdz_est;

        % Check the std and PV.
        delta_dIdzStd(num) = std(delta_dIdz(~isnan(delta_dIdz)));
        delta_dIdzPV(num) = range(delta_dIdz(~isnan(delta_dIdz)));

        if num>1
            OptStd_last = OptStd;
        elseif num==1
            OptStd_last = 0;
        end

        % TIE_solver
        [phi_comp, ~ ] = TIE_solver_meanI(delta_dIdz,I0z,pixelsize,k,RegPara,IntThr,method);
        % Check the std and PV.
        phi_compStd(num) = std(phi_comp(~isnan(phi_comp)));
        phi_compPV(num) = range(phi_comp(~isnan(phi_comp)));

        OptStd = std(delta_dIdz_est(~isnan(delta_dIdz_est)))./std(phi_comp(~isnan(phi_comp)));
        OptStdDiff(num) = OptStd - OptStd_last;

        if strcmpi(JudgeFlag,'delta_dIdzStd')
            Judge = delta_dIdzStd(num)<=StdThr;
        elseif strcmpi(JudgeFlag,'OptStdDiff')
            Judge = OptStdDiff(num)<=0;
        end
        
        if Judge
            break;
        else
            
            % update to phi_DCT.
            phi_DCT = phi_DCT + phi_comp;
        end

        % Error Calculation.
        phiErr_DCT = RemovePiston(phi_DCT - phi);
    end
    
    % Record the error influnced by the noise.
    phiErrStd(NoiseNum) = std(phiErr_DCT(~isnan(phiErr_DCT))); 
    phiErr_DCT_Array(1:Ny,Nx*(NoiseNum-1)+1:Nx*(NoiseNum-1)+Nx)=phiErr_DCT;
    
    dIdz_Array(1:Ny,Nx*(NoiseNum-1)+1:Nx*(NoiseNum-1)+Nx)=dIdz;
    phi_DCT(~Aperture) = NaN;
    phi_DCT_Array(1:Ny,Nx*(NoiseNum-1)+1:Nx*(NoiseNum-1)+Nx) = phi_DCT;
end

% save NoiseResults.mat NOISES phiErrStd dIdz_Array phi_DCT_Array phiErr_DCT_Array

%%

load NoiseResults.mat;

figure;plot(NOISES,phiErrStd,'s-');

figure;imshow(dIdz_Array,[]);
colormap hsv;

MAX = max(phi_DCT_Array(~isnan(phi_DCT_Array)));
MIN = min(phi_DCT_Array(~isnan(phi_DCT_Array)));
figure;imshow(phi_DCT_Array,[]);
colormap jet;

% figure('Color','k');colorbar('YColor','w','FontSize',24);
% colormap jet;
% caxis([MIN MAX]);

MAX = max(phiErr_DCT_Array(~isnan(phiErr_DCT_Array)));
MIN = min(phiErr_DCT_Array(~isnan(phiErr_DCT_Array)));
if abs(MAX)>abs(MIN)
    M = abs(MAX);
elseif abs(MAX)<=abs(MIN)
    M = abs(MIN);
end
figure;imshow(phiErr_DCT_Array,[-M M]);
colormap(WRGBKColorMap2);

% figure('Color','k');colorbar('YColor','y','FontSize',24);
% caxis([-M M]);
% colormap(WRGBKColorMap2);

